package djmil.cordacheckers;

import java.net.URI;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.util.UriComponentsBuilder;

import djmil.cordacheckers.cordaclient.CordaClient;
import djmil.cordacheckers.cordaclient.HoldingIdentityResolver;
import djmil.cordacheckers.cordaclient.dao.HoldingIdentity;
import djmil.cordacheckers.cordaclient.dao.VirtualNode;
import djmil.cordacheckers.game.PlayerColor;
import djmil.cordacheckers.gameproposal.CreateGameProposal;

import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;


@RestController
@RequestMapping("/gameproposal")
public class GameProposalController {

    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    @GetMapping
    public ResponseEntity<String> findAllUnconsumed(
        @AuthenticationPrincipal ApiUserDetails user
    ) {
        String gpList = cordaClient.listGameProposals(user.getHoldingIdentity());

        return ResponseEntity.ok(gpList);
    }

    // @PostMapping()
    // public  ResponseEntity<String> gameproposalSend(@AuthenticationPrincipal ApiUserDetails user) {
        
    //     return  ResponseEntity.ok("");
    // }
    
    @PostMapping()
    public ResponseEntity<Void> create(
        @AuthenticationPrincipal ApiUserDetails player,
        @RequestBody CreateGameProposal gpRequest,
        UriComponentsBuilder ucb
    ) {
        final HoldingIdentity gpSender = player.getHoldingIdentity();
        // TODO: throw execption with custom type
        final HoldingIdentity gpReceiver = holdingIdentityResolver.getByCommonName(gpRequest.opponentName());
        final PlayerColor gpReceiverColor = PlayerColor.valueOf(gpRequest.opponentColor());

        String newGameProposalUuid = cordaClient.sendGameProposal(
            gpSender,
            gpReceiver,
            gpReceiverColor,
            gpRequest.message()
        );

        URI locationOfNewGameProposal = ucb
            .path("gameproposal/{id}")
            .buildAndExpand(newGameProposalUuid)
            .toUri();

        return ResponseEntity
            .created(locationOfNewGameProposal)
            .build();
    }

}